/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


/*****************************************************************************/
/*                                                                           */
/* Jacket routines for dlopen dlsym dlclose                                  */
/*                                                                           */
/* Reason:  VMS doesn't have a dlopen or dlsym or dlclose                    */
/*          but should but easy to emulate using LIB$FIND_IMAGE_SYMBOL       */
/* FIX:     Use LIB$FIND_IMAGE_SYMBOL to locate the symbol within the RTLS   */
/*                                                                           */
/*****************************************************************************/
#include "vms_jackets.h"
#include <dlfcn.h>
#include <string.h>
#include <descrip.h>
#include <ssdef.h>
#include <stsdef.h>
#include <libfisdef.h>
#include <libdef.h>
#include <starlet.h>
#include <lib$routines.h>
#include <stdio.h>
#include <stdlib.h>
#include <rms.h>
#include <errno.h>
#include <signal.h>

static unsigned int GENERIC_dl_last_error=0;
static unsigned int GENERIC_dl_init=0;
static unsigned int GENERIC_dl_fis_flags=0;
static char *GENERIC_dl_force_sysshare;

unsigned int GENERIC_dl_handler( void *__sigarr, void *__mecharr )
{
    /*
    ** Always continue and check for errors in returned status.
    */
    return SS$_CONTINUE;
}

int GENERIC_DLCLOSE(dl_struct *in_handle)
{
    if (!in_handle) return 0;

    if (in_handle->fullname_buf)
	free(in_handle->fullname_buf);

    if (in_handle->given_name)
	free(in_handle->given_name);

    free(in_handle);

    return 0;
}


dl_struct *GENERIC_DLOPEN(const char *fn, int mode)
{
    dl_struct *return_handle;

    char symbol_name[TEMP_STRING_SIZE];
    struct dsc$descriptor_s symbol_desc;
    void *return_addr;
    char temp_string[TEMP_STRING_SIZE];
    char *new_filename;
    int status;

    if (!GENERIC_dl_init) {
	GENERIC_dl_force_sysshare = getenv(VMS_PREFIX "DL_FORCE_SYSSHARE");
	if (getenv(VMS_PREFIX "DL_FIS_FORCUP"))
	    GENERIC_dl_fis_flags = 0;
	else
	    GENERIC_dl_fis_flags = LIB$M_FIS_MIXEDCASE;
	GENERIC_dl_init = 1;
    }


    if (fn && !*fn) {
	GENERIC_dl_last_error = ENOENT;
	return NULL;
    }

    return_handle = malloc (sizeof(dl_struct));
    if (!return_handle) return(NULL); /* out of memory*/

    return_handle->err = 0;
    return_handle->fullname_buf = NULL;
    return_handle->given_name = NULL;

    /*
    ** We need to save our own copy of the file name.
    */
    if (!fn)
       return_handle->given_name = strdup( VMS_PREFIX "NULL_DL_NAME" );
    else
       return_handle->given_name = strdup(fn);

    return_handle->mode = mode;

    if (GENERIC_dl_force_sysshare) {
	/*
	** The old way is the easy way. We are just going to pass whatever we
	** were given to LIB$FIS.
	*/
	return_handle->name_dsc.dsc$b_dtype = DSC$K_DTYPE_T;
	return_handle->name_dsc.dsc$b_class = DSC$K_CLASS_S;
	return_handle->name_dsc.dsc$a_pointer = return_handle->given_name;
	return_handle->name_dsc.dsc$w_length = strlen(return_handle->given_name);
	return_handle->path_dsc = NULL;
    }
    else {
	/*
	** The new way is the "right" way. First we translate whatever we
	** were given to VMS format, then we $PARSE it, and finally from that
	** we set up the name and default name descriptors.
	*/
	struct FAB fab;
	struct NAM nam;
	char nam_esa[NAM$C_MAXRSS];
	char vms_name[TEMP_STRING_SIZE], *pvms, *pexp;
	pvms = GENERIC_EXTERNAL_NAME(return_handle->given_name,&vms_name[0]);
	if (strlen(pvms)==0) {
	    GENERIC_dl_last_error = RMS$_FNF;;
	    GENERIC_DLCLOSE(return_handle); 
	    return(NULL);
	}

	/* Initialise the fab block  */
	memset(&fab,0,sizeof(struct FAB));
	fab = cc$rms_fab;
	fab.fab$l_nam = &nam;
	fab.fab$l_fop = FAB$M_NAM;
	fab.fab$l_fna = pvms;
	fab.fab$b_fns = strlen(pvms);

	/* Initialise the nam block */
	memset(&nam,0,sizeof(struct NAM));
	nam = cc$rms_nam;
	nam.nam$b_nop - NAM$M_SYNCHK;
	nam.nam$l_esa = &nam_esa[0];    /* Expanded string address            */
	nam.nam$b_ess = NAM$C_MAXRSS; /* maximum size of expanded filename str */
                                                             
	return_handle->err = sys$parse(&fab,0,0);
	if (!$VMS_STATUS_SUCCESS(return_handle->err)) {
	    GENERIC_dl_last_error=return_handle->err;
	    GENERIC_DLCLOSE(return_handle); 
	    return(NULL);
	}

	/*
	** Make a copy of the expanded file name.
	*/
	pexp = malloc(nam.nam$b_esl+1);
	strncpy(pexp, nam.nam$l_esa, nam.nam$b_esl);
	pexp[nam.nam$b_esl]='\0';
	return_handle->fullname_buf = pexp;

	/*
	** Set up the descriptors.
	*/
	return_handle->full_dsc.dsc$b_dtype = DSC$K_DTYPE_T;
	return_handle->full_dsc.dsc$b_class = DSC$K_CLASS_S;
	return_handle->full_dsc.dsc$a_pointer = pexp;
	return_handle->full_dsc.dsc$w_length = nam.nam$b_esl;

	return_handle->name_dsc.dsc$b_dtype = DSC$K_DTYPE_T;
	return_handle->name_dsc.dsc$b_class = DSC$K_CLASS_S;
	return_handle->name_dsc.dsc$a_pointer = pexp + 
		(nam.nam$l_name - nam.nam$l_esa);
	return_handle->name_dsc.dsc$w_length = nam.nam$b_name;

	return_handle->path_dsc = &return_handle->full_dsc;

	GENERIC_TRACE("dlopen_Jacket: Mapped:", return_handle->given_name,
			"To:", pexp);
	
    }

    symbol_desc.dsc$b_dtype = DSC$K_DTYPE_T;
    symbol_desc.dsc$b_class = DSC$K_CLASS_S;
    symbol_desc.dsc$a_pointer = &symbol_name[0];
    strcpy(&symbol_name[0],"any_old_name");
    symbol_desc.dsc$w_length=strlen(symbol_name);

    /*
    ** Make sure we can activate this image.
    ** If not, return a NULL pointer.
    */
    VAXC$ESTABLISH(GENERIC_dl_handler);

    return_handle->err =
        lib$find_image_symbol(&return_handle->name_dsc,
				&symbol_desc, &return_addr,
				return_handle->path_dsc,
				GENERIC_dl_fis_flags);

    GENERIC_dl_last_error=return_handle->err;

    if ( (!$VMS_STATUS_SUCCESS(return_handle->err)) &&
         ( return_handle->err != LIB$_KEYNOTFOU)
       )  {
	GENERIC_DLCLOSE(return_handle); /* Free handle space and pointers therein */
	return(NULL);         /* treat as error */
    }    
    return(return_handle);
    
}	
void *GENERIC_DLSYM(dl_struct *in_handle, const char *name)
{
    struct dsc$descriptor_s symbol_desc;
    void *return_addr;

    if (!in_handle) return(NULL);

    symbol_desc.dsc$a_pointer = (char *) name;
    symbol_desc.dsc$b_dtype = DSC$K_DTYPE_T;
    symbol_desc.dsc$b_class = DSC$K_CLASS_S;

    return_addr = 0;
    GENERIC_TRACE("dlsym_Jacket: looking for", name,
		  "in", in_handle->given_name);
    symbol_desc.dsc$w_length=strlen(name);
    VAXC$ESTABLISH(GENERIC_dl_handler);
    in_handle->err =
	lib$find_image_symbol(	&in_handle->name_dsc,
				&symbol_desc, &return_addr,
				in_handle->path_dsc,
				GENERIC_dl_fis_flags);
    GENERIC_dl_last_error=in_handle->err;
    if ($VMS_STATUS_SUCCESS(in_handle->err))
	return(return_addr);

    GENERIC_TRACE("dlsym_Jacket: failed to find", name,
		  "in", in_handle->given_name);
    return(0);

}

char *GENERIC_DLERROR()
{
    char temp_string[TEMP_STRING_SIZE];

    sprintf(temp_string,"OpenVMS status is decimal %d\n",
	GENERIC_dl_last_error);
   
    return(strdup(temp_string));
}

/*
** This routine is no longer used. Its left here just as a placeholder
** so that the layout of the symbol table in the shareable image
** doesn't change. Why is it in this module? Cos this module was the
** only place where we ever used it.
*/

int GENERIC_BUILD_VMSNAME(char *p1, const char *p2) {
    return 0;
}
