/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include "vms_jackets.h"
#include <errno.h>
#include <pwd.h>
#include <unistd.h>
#include <unixlib.h>
#include <rmsdef.h>
#include <ssdef.h>
#include <starlet.h>
#include <stsdef.h>
#include <stdio.h>
#include <types.h>


/*
** This version returns information about all account for which the caller
** has privilege. This means that in order to see all accounts, SYSPRV or
** similar is required.
**
** Note that this routine is NOT re-entrant or threadsafe.
**
** We also have setpwent/endpwent in here and have them "do the
** right thing".
*/

/*
** We want to make sure that we use the jacketed version of gwtpwuid,
** even though we are a jacket ourselved.
*/
#ifndef BUILD_GETPWENT_MAIN
#define getpwuid GENERIC_GETPWUID_JACKET
struct passwd *getpwuid(uid_t);
#endif


unsigned int getpwent_ctx = 0;

/*
** endpwent closes the user database. For us, make sure that the next
** call will return the first user.
*/
void GENERIC_ENDPWENT (void)
{
    int rv;
    rv=sys$finish_rdb(&getpwent_ctx);
}

/*
** setpwent rewinds the user database. For us, its equivalent to calling
** endpwent.
*/
void GENERIC_SETPWENT (void)
{
    GENERIC_ENDPWENT();
}

struct passwd *GENERIC_GETPWENT (void)
{
    /*
    ** If we returned NULL last time, then that effectively was the end of
    ** the list, so this time we return the current user. If we returned
    ** the user last time, then this time we return NULL (end of list).
    */
    struct passwd *pwd;
    unsigned int id;
    int rv, again;

    do {
	/*
	** Assume that we won't need to try and get another one.
	*/
	again=0;
	/*
	** Call IDTOASC to get the next identifier.
	*/
	rv=sys$idtoasc(-1,NULL,NULL,&id,NULL,&getpwent_ctx);
	if ($VMS_STATUS_SUCCESS(rv)) {
	    /*
	    ** We got an ID, so try and get the account details.
	    */
	    pwd = getpwuid(id);
	    if (pwd) {
		/*
		** That worked, return the passed struct to our caller.
		*/
		return pwd;
	    }
	    else {
		/*
		** Something went wrong.
		**
		** If the identifier does not belong to an account, then we
		** get back RMS$_RNF. If we don't have priv for this account,
		** we get back SS$_NOSYSPRV.
		** These are OK, just go on to the next one.
		**
		** Any other error is a hard error.
		*/
		if ( (errno == EVMSERR) && 
		     ((vaxc$errno == RMS$_RNF) | (vaxc$errno == SS$_NOSYSPRV))
		   )
		    again=1;
	    }
	}
    } while (again);

    /*
    ** Some or other error back from ISTOASC. SS$_NOSUCHID is what
    ** we expect when we're out of ID's. Anything else is a real error.
    */
    if (rv == SS$_NOSUCHID) {
	errno=0;
    }
    else {
	errno=EVMSERR;
	vaxc$errno=rv;
    }

    /*
    ** Either way, we're done for this session.
    */
    GENERIC_ENDPWENT();
    return NULL;
}

#ifdef BUILD_GETPWENT_MAIN
int main() {
    unsigned int uicg, uicm;
    struct passwd *pwd = NULL;

    while (pwd=GENERIC_GETPWENT()) {
	uicg = (pwd->pw_uid & 0xFFFF0000) >> 16;
	uicm = (pwd->pw_uid & 0x0000FFFF);
	printf("%s, [%o,%o], %s\n",
	    pwd->pw_name,uicg,uicm,pwd->pw_dir);
    }

    printf("\nRestart\n");

    while (pwd=GENERIC_GETPWENT()) {
	uicg = (pwd->pw_uid & 0xFFFF0000) >> 16;
	uicm = (pwd->pw_uid & 0x0000FFFF);
	printf("%s, [%o,%o], %s\n",
	    pwd->pw_name,uicg,uicm,pwd->pw_dir);
    }

    return 0;
}
#endif
