/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include "vms_jackets.h"
#include <sys/statfs.h>
#include <limits.h>
#include <namdef.h>
#ifdef __ALPHA
#include <pagedef.h>
#else
#define IOC$C_DISK_BLKSIZ 512
#endif
#include <stdio.h>

/*
** One day this routine might do something more correctly, like return
** the amount of disk space on the device where name resides, or the user's
** remaining quota on that device. But for now, just return something
** so that we don't accvio.
*/

int GENERIC_STATFS(const char *name, struct statfs *sb) {

    /* file system block size */
    sb->f_bsize = IOC$C_DISK_BLKSIZ;

    /* fundamental filesystem block size */
    sb->f_frsize = IOC$C_DISK_BLKSIZ;

    /* total number of blocks on file system in units of f_frsize */
    sb->f_blocks = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* total number of free blocks */
    sb->f_bfree = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* number of free blocks available to non-privileged process */
    sb->f_bavail = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* total number of file serial numbers */
    sb->f_files = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* total number of free file serial numbers */
    sb->f_ffree = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* number of file serial numbers available to non-privileged process */
    sb->f_favail = ULONG_MAX / IOC$C_DISK_BLKSIZ;

    /* file system id */
    sb->f_fsid= 1;

    /* bit mask of f_flag values */
    sb->f_flag = 0;

    /* maximum filename length */
    sb->f_namemax = NAM$C_MAXRSS;

    return 0;
}

#ifdef STATFS_MAIN
int main (int argc, char *argv[]) {
    struct statfs blk;
    int rv;

    rv = statfs(argv[1],&blk);
    printf("statfs(%s) returned %d\n",argv[1],rv);
    printf("  f_bsize   :  %d\n",blk.f_bsize);
    printf("  f_frsize  :  %d\n",blk.f_frsize);
    printf("  f_blocks  :  %d\n",blk.f_blocks);
    printf("  f_bfree   :  %d\n",blk.f_bfree);
    printf("  f_bavail  :  %d\n",blk.f_bavail);
    printf("  f_files   :  %d\n",blk.f_files);
    printf("  f_ffree   :  %d\n",blk.f_ffree);
    printf("  f_favail  :  %d\n",blk.f_favail);
    printf("  f_fsid    :  %d\n",blk.f_fsid);
    printf("  f_flag    :  %d\n",blk.f_flag);
    printf("  f_namemax :  %d\n",blk.f_namemax);

    return 0;
}
#endif
