/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include <string.h>
#include <unixlib.h>
#include <namdef.h>
#include <stdlib.h>
#include <starlet.h>
#include <fabdef.h>

/*
** Routine to determine if a filespec is VMS format or not
*/
int GENERIC_IS_VALID_VMS_FILE_SPEC(char *file_name)
{

struct NAM nam;
struct FAB fab;
int stat;

    fab = cc$rms_fab;
    nam = cc$rms_nam;
    fab.fab$l_fna = file_name;
    fab.fab$b_fns = strlen( file_name );
    nam.nam$b_nop = NAM$M_SYNCHK; /* This means we do not want to I/O
                                     to verify file exist. only parse name */
    fab.fab$l_nam = & nam;

    stat = sys$parse( &fab );
    if ( ! (stat&1 ) )
	return 0;
    else
	return 1;

}


/*
**  Action routine which creates a dynamic memory copy of the callback name
*/
static char *action_buffer = NULL;

static int action_routine (char *string, int unused) {

   if (action_buffer) 
   {
       strcpy(action_buffer,string);
   } 
   else 
   {
       action_buffer = strdup(string);
   }

   return (0);
}


char * GENERIC_EXTERNAL_NAME (const char *unixPtr, char *optBuffer) 
{
   char local_unix [NAM$C_MAXRSS+1];
   size_t offset = strlen(unixPtr) - 1;

   if( GENERIC_IS_VALID_VMS_FILE_SPEC( (char *)unixPtr ) )
   {  
       if( optBuffer )
       {
           strcpy( optBuffer, unixPtr ); /* hopefully user given enough space*/
           return optBuffer;
       }
       else
       {
           action_buffer = (char *)malloc( strlen(unixPtr) + 1 );
           strcpy( action_buffer, unixPtr );
           return action_buffer;
       }
   }
   else
   {
       strcpy (local_unix, unixPtr);
       while (local_unix[offset] == '/') local_unix[offset--] = '\0';

       if (optBuffer) 
       {
          action_buffer = optBuffer;
          optBuffer[0]='\0'; /* in case no translations */
       } 
       else 
       {
       action_buffer = NULL;
       }

       decc$to_vms(local_unix, action_routine, -1, 1);
       /* decc$to_vms(local_unix, action_routine, 0, 1); */
      return (action_buffer);
   }
}


char * GENERIC_EXTERNAL_DIR_NAME (const char *unixPtr, char *optBuffer) 
{

   char local_unix [NAM$C_MAXRSS+1];
   size_t offset = strlen(unixPtr) - 1;

   if( GENERIC_IS_VALID_VMS_FILE_SPEC( (char *)unixPtr ) )
   {  
       if( optBuffer )
       {
           strcpy( optBuffer, unixPtr ); /* hopefully user given enough space*/
           return optBuffer;
       }
       else
       {
           action_buffer = (char *)malloc( strlen(unixPtr) + 1 );
           strcpy( action_buffer, unixPtr );
           return action_buffer;
       }
   }
   else
   {
      strcpy (local_unix, unixPtr);
      while (local_unix[offset] == '/') local_unix[offset--] = '\0';

      if (optBuffer) 
      {
          action_buffer = optBuffer;
      } 
      else 
      {
          action_buffer = NULL;
      }

      decc$to_vms(local_unix, action_routine, -1, 2);
      /* decc$to_vms(local_unix, action_routine, 0, 2); */
      return (action_buffer);
   }
}


