/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


/*
** UTILS.C
**
** This is the place we put any so-called utility routines. That is, routines
** that are used by one or more of the jacket routines.
*/

#include <string.h>


/*
** generic_tidy_spec - simplifies a filespec so that the checks for perform
** later have a better chance of working as expected. For example,
** "/dka0/user/fred/.." is really "/dka0/user" and 
** "dka0/user/fred////../flintstone" is really "dka0/user/flintstone"
**
** It is assumed that the destination string is already allocated and is
** at least the size of the source string (we only ever shorten strings).
*/
void generic_tidy_spec(char *to, const char *from) {
    char *p,*pp;
    int len;

    /*
    ** Start off my moving the string from source to dest.
    */
    strcpy(to,from);

    /*
    ** Change any "//" to "/".
    */
    while (p=strstr(to,"//")) {
	len=strlen(p+1);
	memmove(p,p+1,len);
	p[len]='\0';
    }

    /*
    ** Change any "/./" to "/".
    */
    while (p=strstr(to,"/./")) {
	len=strlen(p+2);
	memmove(p,p+2,len);
	p[len]='\0';
    }

    /*
    ** Change any "/foo/.." to "".
    */
    while (p=strstr(to+1,"/..")) {
	*p='\0';
	pp=strrchr(to,'/');
	if (pp) {
	    p=p+3;
	    len=strlen(p);
	    memmove(pp,p,len);
	    pp[len]='\0';
	}
	else
	    *p='/';
    }

    /*
    ** Change a leading "/.." to "".
    */
    while (!strncmp(to,"/..",3)) {
	p=to;
	len=strlen(p+3);
	memmove(p,p+3,len);
	p[len]='\0';
    }

#if 0
    /*
    ** Change a trailing "/." (which is modified to "/_" by the file
    ** parser routines) to "/".
    */
    p=to+strlen(to)-2;
    if (!strncmp(p,"/_",2)) {
	p[1]='\0';
    }
#endif

    /*
    ** If we have nothing left, then assume "/". This could happen with
    ** something like "/dka0/users/../.." which really means root.
    */
    if (!strlen(to))
	strcpy(to,"/");

}

/*
** generic_absone - this routine returns true if the file/dirspec passed in
** is an absolute spec containing one (and only one) term.
*/
int generic_absone(const char *spec) {

    int len;
    char *p;

    /*
    ** Check that we weren't called with a null.
    */
    if (spec == NULL)
	return 0;

    /*
    ** If the spec doesn't have at least two chars, then it can't be right.
    */
    len = strlen(spec);
    if (len < 2)
	return 0;

    /*
    ** If the spec doesn't start with a "/", then its not absolute.
    */
    if (spec[0] != '/')
	return 0;

    /*
    ** If only one slash, then that's OK - eg. /dka0
    */
    p=strchr(&spec[1],'/');
    if (!p)
	return 1;

    /*
    ** If a second slash, and it has nothing after it, then
    ** that's OK too - eg. /dka0/
    */
    if (p[1] == '\0')
	return 1;

    /*
    ** Didn't match any of our tests.
    */
    return 0;
}
