/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


/****************************************************************************
**
**  MODULE DESCRIPTION:
**
**      writev for OpenVMS
**
**      Performs a write from a set of buffers. Continues writing from the
**      buffers until all done, or until one of them is not completely
**      written. Always returns the number of bytes written.
**
**	We have to be careful to correctly emulate the way writev handles
**	interrupted operations. Here's what the spec says about both
**	write and writev:
**
**		If write() is interrupted by a signal before it writes any
**		data, it will return -1 with errno set to [EINTR].
**
**		If write() is interrupted by a signal after it successfully
**		writes some data, it will return the number of bytes written. 
**
**	If one of the segments contains a large amount of data (more than
**	MAXBUF), then the write will fails with errno=EVMSERR and
**	vaxc$errno=SS$_IVBUFLEN. For this reason for use our own
**	GENERIC_WRITE since this routine knows how to deal with the
**	problem.
*/

#include <uio.h>
#include <unistd.h>

ssize_t GENERIC_WRITE(int fildes, const void *buf, size_t nbyte);

int GENERIC_WRITEV(int fd, const struct iovec *iov, int count)
{
    int i, bytes, total=0;

    for (i=0; i<count; i++) {
        if (iov[i].iov_len != 0) {
            bytes = GENERIC_WRITE(fd, iov[i].iov_base, iov[i].iov_len);
            if (bytes >= 0) {
                total+=bytes;
                if (bytes != iov[i].iov_len)
                    break;
            }
            else {
		if (total == 0)
		    total=-1;
                break;
	    }
        }
    }
    return total;
}
